@@ -1,14 +1,19 @@
 from numbers import Real
 from typing import (List,
-                    Tuple)
+                    Tuple,
+                    TypeVar)
 
 from hypothesis.strategies import SearchStrategy
 
+Domain = TypeVar('Domain')
+Range = TypeVar('Range')
 Strategy = SearchStrategy
 Coordinate = Real
+BoundingBox = Tuple[Coordinate, Coordinate, Coordinate, Coordinate]
 Point = Tuple[Coordinate, Coordinate]
 Segment = Tuple[Point, Point]
-BoundingBox = Tuple[Coordinate, Coordinate, Coordinate, Coordinate]
+Multisegment = List[Segment]
 Contour = List[Point]
 Polyline = List[Point]
-Polygon = Tuple[Contour, List[Contour]]
+Multicontour = List[Contour]
+Polygon = Tuple[Contour, Multicontour]
