@@ -1,6 +1,7 @@
 import warnings
 from functools import partial
-from itertools import (cycle,
+from itertools import (accumulate,
+                       cycle,
                        groupby)
 from random import Random
 from typing import (List,
@@ -13,21 +14,25 @@ from hypothesis.errors import HypothesisWarning
 
 from .core.contracts import (is_contour_non_convex,
                              is_contour_strict,
+                             is_multisegment_valid,
                              points_do_not_lie_on_the_same_line)
 from .core.utils import (Orientation,
                          orientation)
 from .hints import (BoundingBox,
                     Contour,
                     Coordinate,
+                    Multicontour,
+                    Multisegment,
                     Point,
                     Polygon,
                     Polyline,
                     Segment,
                     Strategy)
 from .utils import (constrict_convex_hull_size,
+                    contour_to_segments,
                     pack,
                     sort_pair,
-                    to_concave_contour,
+                    to_contour,
                     to_convex_contour,
                     to_convex_hull,
                     to_polygon,
@@ -181,6 +186,131 @@ def segments(x_coordinates: Strategy[Coordinate],
             .filter(non_degenerate_segment))
 
 
+EMPTY_MULTISEGMENT_SIZE = 0
+
+
+def multisegments(x_coordinates: Strategy[Coordinate],
+                  y_coordinates: Optional[Strategy[Coordinate]] = None,
+                  *,
+                  min_size: int = EMPTY_MULTISEGMENT_SIZE,
+                  max_size: Optional[int] = None) -> Strategy[Multisegment]:
+    """
+    Returns a strategy for multisegments.
+    Multisegment is a possibly empty sequence of segments
+    such that any pair of them do not cross/overlap each other.
+
+    >>> from hypothesis import strategies
+    >>> from hypothesis_geometry import planar
+
+    For same coordinates' domain:
+
+    >>> min_coordinate, max_coordinate = -1., 1.
+    >>> coordinates_type = float
+    >>> coordinates = strategies.floats(min_coordinate, max_coordinate,
+    ...                                 allow_infinity=False,
+    ...                                 allow_nan=False)
+    >>> min_size, max_size = 5, 10
+    >>> multisegments = planar.multisegments(coordinates,
+    ...                                      min_size=min_size,
+    ...                                      max_size=max_size)
+    >>> multisegment = multisegments.example()
+    >>> isinstance(multisegment, list)
+    True
+    >>> min_size <= len(multisegment) <= max_size
+    True
+    >>> all(isinstance(segment, tuple) for segment in multisegment)
+    True
+    >>> all(isinstance(endpoint, tuple)
+    ...     for segment in multisegment
+    ...     for endpoint in segment)
+    True
+    >>> all(len(segment) == 2 for segment in multisegment)
+    True
+    >>> all(len(endpoint) == 2
+    ...     for segment in multisegment
+    ...     for endpoint in segment)
+    True
+    >>> all(isinstance(coordinate, coordinates_type)
+    ...     for segment in multisegment
+    ...     for endpoint in segment
+    ...     for coordinate in endpoint)
+    True
+    >>> all(min_coordinate <= coordinate <= max_coordinate
+    ...     for segment in multisegment
+    ...     for endpoint in segment
+    ...     for coordinate in endpoint)
+    True
+
+    For different coordinates' domains:
+
+    >>> min_x_coordinate, max_x_coordinate = -1., 1.
+    >>> min_y_coordinate, max_y_coordinate = 10., 100.
+    >>> coordinates_type = float
+    >>> x_coordinates = strategies.floats(min_x_coordinate, max_x_coordinate,
+    ...                                   allow_infinity=False,
+    ...                                   allow_nan=False)
+    >>> y_coordinates = strategies.floats(min_y_coordinate, max_y_coordinate,
+    ...                                   allow_infinity=False,
+    ...                                   allow_nan=False)
+    >>> min_size, max_size = 5, 10
+    >>> multisegments = planar.multisegments(x_coordinates, y_coordinates,
+    ...                                      min_size=min_size,
+    ...                                      max_size=max_size)
+    >>> multisegment = multisegments.example()
+    >>> isinstance(multisegment, list)
+    True
+    >>> min_size <= len(multisegment) <= max_size
+    True
+    >>> all(isinstance(segment, tuple) for segment in multisegment)
+    True
+    >>> all(isinstance(endpoint, tuple)
+    ...     for segment in multisegment
+    ...     for endpoint in segment)
+    True
+    >>> all(len(segment) == 2 for segment in multisegment)
+    True
+    >>> all(len(endpoint) == 2
+    ...     for segment in multisegment
+    ...     for endpoint in segment)
+    True
+    >>> all(isinstance(coordinate, coordinates_type)
+    ...     for segment in multisegment
+    ...     for endpoint in segment
+    ...     for coordinate in endpoint)
+    True
+    >>> all(min_x_coordinate <= endpoint_x <= max_x_coordinate
+    ...     and min_y_coordinate <= endpoint_y <= max_y_coordinate
+    ...     for segment in multisegment
+    ...     for endpoint_x, endpoint_y in segment)
+    True
+    """
+    _validate_sizes(min_size, max_size, EMPTY_MULTISEGMENT_SIZE)
+    min_size = max(min_size, EMPTY_MULTISEGMENT_SIZE)
+    if min_size >= TRIANGULAR_CONTOUR_SIZE:
+        def multisegment_to_slices(multisegment: Multisegment
+                                   ) -> Strategy[Multisegment]:
+            return strategies.builds(cut,
+                                     strategies.permutations(multisegment),
+                                     strategies.integers(min_size,
+                                                         len(multisegment)))
+
+        def cut(multisegment: Multisegment, limit: int) -> Multisegment:
+            return (multisegment[:limit]
+                    if limit < len(multisegment)
+                    else multisegment)
+
+        return (contours(x_coordinates, y_coordinates,
+                         min_size=min_size,
+                         max_size=max_size)
+                .map(contour_to_segments)
+                .flatmap(multisegment_to_slices))
+    else:
+        return (strategies.lists(segments(x_coordinates, y_coordinates),
+                                 min_size=min_size,
+                                 max_size=max_size)
+                .filter(is_multisegment_valid))
+
+
 MIN_POLYLINE_SIZE = 2
 
 
@@ -224,13 +354,13 @@ def polylines(x_coordinates: Strategy[Coordinate],
     True
     >>> all(len(vertex) == 2 for vertex in polyline)
     True
-    >>> all(all(isinstance(coordinate, coordinates_type)
-    ...         for coordinate in vertex)
-    ...     for vertex in polyline)
+    >>> all(isinstance(coordinate, coordinates_type)
+    ...     for vertex in polyline
+    ...     for coordinate in vertex)
     True
-    >>> all(all(min_coordinate <= coordinate <= max_coordinate
-    ...         for coordinate in vertex)
-    ...     for vertex in polyline)
+    >>> all(min_coordinate <= coordinate <= max_coordinate
+    ...     for vertex in polyline
+    ...     for coordinate in vertex)
     True
 
     For different coordinates' domains:
@@ -257,9 +387,9 @@ def polylines(x_coordinates: Strategy[Coordinate],
     True
     >>> all(len(vertex) == 2 for vertex in polyline)
     True
-    >>> all(all(isinstance(coordinate, coordinates_type)
-    ...         for coordinate in vertex)
-    ...     for vertex in polyline)
+    >>> all(isinstance(coordinate, coordinates_type)
+    ...     for vertex in polyline
+    ...     for coordinate in vertex)
     True
     >>> all(min_x_coordinate <= vertex_x <= max_x_coordinate
     ...     and min_y_coordinate <= vertex_y <= max_y_coordinate
@@ -350,13 +480,13 @@ def contours(x_coordinates: Strategy[Coordinate],
     True
     >>> all(len(vertex) == 2 for vertex in contour)
     True
-    >>> all(all(isinstance(coordinate, coordinates_type)
-    ...         for coordinate in vertex)
-    ...     for vertex in contour)
+    >>> all(isinstance(coordinate, coordinates_type)
+    ...     for vertex in contour
+    ...     for coordinate in vertex)
     True
-    >>> all(all(min_coordinate <= coordinate <= max_coordinate
-    ...         for coordinate in vertex)
-    ...     for vertex in contour)
+    >>> all(min_coordinate <= coordinate <= max_coordinate
+    ...     for vertex in contour
+    ...     for coordinate in vertex)
     True
 
     For different coordinates' domains:
@@ -383,9 +513,9 @@ def contours(x_coordinates: Strategy[Coordinate],
     True
     >>> all(len(vertex) == 2 for vertex in contour)
     True
-    >>> all(all(isinstance(coordinate, coordinates_type)
-    ...         for coordinate in vertex)
-    ...     for vertex in contour)
+    >>> all(isinstance(coordinate, coordinates_type)
+    ...     for vertex in contour
+    ...     for coordinate in vertex)
     True
     >>> all(min_x_coordinate <= vertex_x <= max_x_coordinate
     ...     and min_y_coordinate <= vertex_y <= max_y_coordinate
@@ -446,13 +576,13 @@ def convex_contours(x_coordinates: Strategy[Coordinate],
     True
     >>> all(len(vertex) == 2 for vertex in contour)
     True
-    >>> all(all(isinstance(coordinate, coordinates_type)
-    ...         for coordinate in vertex)
-    ...     for vertex in contour)
+    >>> all(isinstance(coordinate, coordinates_type)
+    ...     for vertex in contour
+    ...     for coordinate in vertex)
     True
-    >>> all(all(min_coordinate <= coordinate <= max_coordinate
-    ...         for coordinate in vertex)
-    ...     for vertex in contour)
+    >>> all(min_coordinate <= coordinate <= max_coordinate
+    ...     for vertex in contour
+    ...     for coordinate in vertex)
     True
 
     For different coordinates' domains:
@@ -479,9 +609,9 @@ def convex_contours(x_coordinates: Strategy[Coordinate],
     True
     >>> all(len(vertex) == 2 for vertex in contour)
     True
-    >>> all(all(isinstance(coordinate, coordinates_type)
-    ...         for coordinate in vertex)
-    ...     for vertex in contour)
+    >>> all(isinstance(coordinate, coordinates_type)
+    ...     for vertex in contour
+    ...     for coordinate in vertex)
     True
     >>> all(min_x_coordinate <= vertex_x <= max_x_coordinate
     ...     and min_y_coordinate <= vertex_y <= max_y_coordinate
@@ -553,13 +683,13 @@ def concave_contours(x_coordinates: Strategy[Coordinate],
     True
     >>> all(len(vertex) == 2 for vertex in contour)
     True
-    >>> all(all(isinstance(coordinate, coordinates_type)
-    ...         for coordinate in vertex)
-    ...     for vertex in contour)
+    >>> all(isinstance(coordinate, coordinates_type)
+    ...     for vertex in contour
+    ...     for coordinate in vertex)
     True
-    >>> all(all(min_coordinate <= coordinate <= max_coordinate
-    ...         for coordinate in vertex)
-    ...     for vertex in contour)
+    >>> all(min_coordinate <= coordinate <= max_coordinate
+    ...     for vertex in contour
+    ...     for coordinate in vertex)
     True
 
     For different coordinates' domains:
@@ -586,9 +716,9 @@ def concave_contours(x_coordinates: Strategy[Coordinate],
     True
     >>> all(len(vertex) == 2 for vertex in contour)
     True
-    >>> all(all(isinstance(coordinate, coordinates_type)
-    ...         for coordinate in vertex)
-    ...     for vertex in contour)
+    >>> all(isinstance(coordinate, coordinates_type)
+    ...     for vertex in contour
+    ...     for coordinate in vertex)
     True
     >>> all(min_x_coordinate <= vertex_x <= max_x_coordinate
     ...     and min_y_coordinate <= vertex_y <= max_y_coordinate
@@ -612,7 +742,7 @@ def concave_contours(x_coordinates: Strategy[Coordinate],
                              unique=True)
             .filter(points_do_not_lie_on_the_same_line)
             .flatmap(to_points_with_sizes)
-            .map(pack(to_concave_contour))
+            .map(pack(to_contour))
             .filter(partial(_has_valid_size,
                             min_size=min_size,
                             max_size=max_size))
@@ -651,13 +781,13 @@ def triangular_contours(x_coordinates: Strategy[Coordinate],
     True
     >>> all(len(vertex) == 2 for vertex in contour)
     True
-    >>> all(all(isinstance(coordinate, coordinates_type)
-    ...         for coordinate in vertex)
-    ...     for vertex in contour)
+    >>> all(isinstance(coordinate, coordinates_type)
+    ...     for vertex in contour
+    ...     for coordinate in vertex)
     True
-    >>> all(all(min_coordinate <= coordinate <= max_coordinate
-    ...         for coordinate in vertex)
-    ...     for vertex in contour)
+    >>> all(min_coordinate <= coordinate <= max_coordinate
+    ...     for vertex in contour
+    ...     for coordinate in vertex)
     True
 
     For different coordinates' domains:
@@ -681,9 +811,9 @@ def triangular_contours(x_coordinates: Strategy[Coordinate],
     True
     >>> all(len(vertex) == 2 for vertex in contour)
     True
-    >>> all(all(isinstance(coordinate, coordinates_type)
-    ...         for coordinate in vertex)
-    ...     for vertex in contour)
+    >>> all(isinstance(coordinate, coordinates_type)
+    ...     for vertex in contour
+    ...     for coordinate in vertex)
     True
     >>> all(min_x_coordinate <= vertex_x <= max_x_coordinate
     ...     and min_y_coordinate <= vertex_y <= max_y_coordinate
@@ -738,13 +868,13 @@ def rectangular_contours(x_coordinates: Strategy[Coordinate],
     True
     >>> all(len(vertex) == 2 for vertex in contour)
     True
-    >>> all(all(isinstance(coordinate, coordinates_type)
-    ...         for coordinate in vertex)
-    ...     for vertex in contour)
+    >>> all(isinstance(coordinate, coordinates_type)
+    ...     for vertex in contour
+    ...     for coordinate in vertex)
     True
-    >>> all(all(min_coordinate <= coordinate <= max_coordinate
-    ...         for coordinate in vertex)
-    ...     for vertex in contour)
+    >>> all(min_coordinate <= coordinate <= max_coordinate
+    ...     for vertex in contour
+    ...     for coordinate in vertex)
     True
 
     For different coordinates' domains:
@@ -768,9 +898,9 @@ def rectangular_contours(x_coordinates: Strategy[Coordinate],
     True
     >>> all(len(vertex) == 2 for vertex in contour)
     True
-    >>> all(all(isinstance(coordinate, coordinates_type)
-    ...         for coordinate in vertex)
-    ...     for vertex in contour)
+    >>> all(isinstance(coordinate, coordinates_type)
+    ...     for vertex in contour
+    ...     for coordinate in vertex)
     True
     >>> all(min_x_coordinate <= vertex_x <= max_x_coordinate
     ...     and min_y_coordinate <= vertex_y <= max_y_coordinate
@@ -871,7 +1001,180 @@ def bounding_boxes(x_coordinates: Strategy[Coordinate],
                              .map(sort_pair))
 
 
-EMPTY_HOLES_SIZE = 0
+EMPTY_MULTICONTOUR_SIZE = 0
+
+
+def multicontours(x_coordinates: Strategy[Coordinate],
+                  y_coordinates: Optional[Strategy[Coordinate]] = None,
+                  *,
+                  min_size: int = EMPTY_MULTICONTOUR_SIZE,
+                  max_size: Optional[int] = None,
+                  min_contour_size: int = TRIANGULAR_CONTOUR_SIZE,
+                  max_contour_size: Optional[int] = None
+                  ) -> Strategy[Multicontour]:
+    """
+    Returns a strategy for multicontours.
+    Multicontour is a possibly empty sequence of non-crossing
+    and non-overlapping contours.
+
+    :param x_coordinates: strategy for vertices' x-coordinates.
+    :param y_coordinates:
+        strategy for vertices' y-coordinates,
+        ``None`` for reusing x-coordinates strategy.
+    :param min_size: lower bound for size.
+    :param max_size: upper bound for size, ``None`` for unbound.
+    :param min_contour_size: lower bound for contour size.
+    :param max_contour_size:
+        upper bound for contour size, ``None`` for unbound.
+
+    >>> from hypothesis import strategies
+    >>> from hypothesis_geometry import planar
+
+    For same coordinates' domain:
+
+    >>> min_coordinate, max_coordinate = -1., 1.
+    >>> coordinates_type = float
+    >>> coordinates = strategies.floats(min_coordinate, max_coordinate,
+    ...                                 allow_infinity=False,
+    ...                                 allow_nan=False)
+    >>> min_size, max_size = 5, 10
+    >>> min_contour_size, max_contour_size = 3, 5
+    >>> multicontours = planar.multicontours(coordinates,
+    ...                                      min_size=min_size,
+    ...                                      max_size=max_size,
+    ...                                      min_contour_size=min_contour_size,
+    ...                                      max_contour_size=max_contour_size)
+    >>> multicontour = multicontours.example()
+    >>> isinstance(multicontour, list)
+    True
+    >>> all(isinstance(contour, list) for contour in multicontour)
+    True
+    >>> min_size <= len(multicontour) <= max_size
+    True
+    >>> all(min_contour_size <= len(contour) <= max_contour_size
+    ...     for contour in multicontour)
+    True
+    >>> all(isinstance(vertex, tuple)
+    ...     for contour in multicontour
+    ...     for vertex in contour)
+    True
+    >>> all(len(vertex) == 2 for contour in multicontour for vertex in contour)
+    True
+    >>> all(isinstance(coordinate, coordinates_type)
+    ...     for contour in multicontour
+    ...     for vertex in contour
+    ...     for coordinate in vertex)
+    True
+    >>> all(min_coordinate <= coordinate <= max_coordinate
+    ...     for contour in multicontour
+    ...     for vertex in contour
+    ...     for coordinate in vertex)
+    True
+
+    For different coordinates' domains:
+
+    >>> min_x_coordinate, max_x_coordinate = -1., 1.
+    >>> min_y_coordinate, max_y_coordinate = 10., 100.
+    >>> coordinates_type = float
+    >>> x_coordinates = strategies.floats(min_x_coordinate, max_x_coordinate,
+    ...                                   allow_infinity=False,
+    ...                                   allow_nan=False)
+    >>> y_coordinates = strategies.floats(min_y_coordinate, max_y_coordinate,
+    ...                                   allow_infinity=False,
+    ...                                   allow_nan=False)
+    >>> min_size, max_size = 5, 10
+    >>> min_contour_size, max_contour_size = 3, 5
+    >>> multicontours = planar.multicontours(x_coordinates, y_coordinates,
+    ...                            min_size=min_size,
+    ...                            max_size=max_size,
+    ...                            min_contour_size=min_contour_size,
+    ...                            max_contour_size=max_contour_size)
+    >>> multicontour = multicontours.example()
+    >>> isinstance(multicontour, list)
+    True
+    >>> all(isinstance(contour, list) for contour in multicontour)
+    True
+    >>> min_size <= len(multicontour) <= max_size
+    True
+    >>> all(min_contour_size <= len(contour) <= max_contour_size
+    ...     for contour in multicontour)
+    True
+    >>> all(isinstance(vertex, tuple)
+    ...     for contour in multicontour
+    ...     for vertex in contour)
+    True
+    >>> all(len(vertex) == 2 for contour in multicontour for vertex in contour)
+    True
+    >>> all(isinstance(coordinate, coordinates_type)
+    ...     for contour in multicontour
+    ...     for vertex in contour
+    ...     for coordinate in vertex)
+    True
+    >>> all(min_x_coordinate <= vertex_x <= max_x_coordinate
+    ...     and min_y_coordinate <= vertex_y <= max_y_coordinate
+    ...     for contour in multicontour
+    ...     for vertex_x, vertex_y in contour)
+    True
+    """
+    _validate_sizes(min_size, max_size, EMPTY_MULTICONTOUR_SIZE)
+    _validate_sizes(min_contour_size, max_contour_size,
+                    TRIANGULAR_CONTOUR_SIZE,
+                    'min_contour_size', 'max_contour_size')
+    min_size, min_contour_size = (max(min_size, EMPTY_MULTICONTOUR_SIZE),
+                                  max(min_contour_size,
+                                      TRIANGULAR_CONTOUR_SIZE))
+
+    def to_vertices_with_sizes(vertices: List[Point]
+                               ) -> Strategy[Tuple[List[Point], List[int]]]:
+        return strategies.tuples(strategies.just(vertices),
+                                 to_sizes(len(vertices)))
+
+    def to_sizes(limit: int) -> Strategy[List[int]]:
+        return (strategies.integers(min_size, limit // min_contour_size)
+                .flatmap(partial(_to_sizes,
+                                 min_element_size=min_contour_size,
+                                 limit=limit)))
+
+    def _to_sizes(size: int,
+                  min_element_size: int,
+                  limit: int) -> Strategy[List[int]]:
+        if not size:
+            return strategies.builds(list)
+        max_sizes = [min_element_size] * size
+        indices = cycle(range(size))
+        for _ in range(limit - size * min_element_size):
+            max_sizes[next(indices)] += 1
+        sizes_ranges = [range(min_element_size, max_element_size + 1)
+                        for max_element_size in max_sizes]
+        return (strategies.permutations([strategies.sampled_from(sizes_range)
+                                         for sizes_range in sizes_ranges])
+                .flatmap(pack(strategies.tuples))
+                .map(list))
+
+    def to_contours(vertices: List[Point], sizes: List[int]) -> List[Contour]:
+        vertices = sorted(vertices)
+        return [to_contour(vertices[offset:offset + size], size)
+                for offset, size in zip(accumulate([0] + sizes), sizes)]
+
+    def has_valid_sizes(multicontour: Multicontour) -> bool:
+        return (_has_valid_size(multicontour,
+                                min_size=min_size,
+                                max_size=max_size)
+                and all(_has_valid_size(contour,
+                                        min_size=min_contour_size,
+                                        max_size=max_contour_size)
+                        for contour in multicontour))
+
+    return (strategies.lists(points(x_coordinates, y_coordinates),
+                             min_size=min_size * min_contour_size,
+                             max_size=(None
+                                       if (max_size is None
+                                           or max_contour_size is None)
+                                       else max_size * max_contour_size),
+                             unique=True)
+            .flatmap(to_vertices_with_sizes)
+            .map(pack(to_contours))
+            .filter(has_valid_sizes))
 
 
 def polygons(x_coordinates: Strategy[Coordinate],
@@ -879,15 +1182,14 @@ def polygons(x_coordinates: Strategy[Coordinate],
              *,
              min_size: int = TRIANGULAR_CONTOUR_SIZE,
              max_size: Optional[int] = None,
-             min_holes_size: int = EMPTY_HOLES_SIZE,
+             min_holes_size: int = EMPTY_MULTICONTOUR_SIZE,
              max_holes_size: Optional[int] = None,
              min_hole_size: int = TRIANGULAR_CONTOUR_SIZE,
              max_hole_size: Optional[int] = None) -> Strategy[Polygon]:
     """
     Returns a strategy for polygons.
     Polygon is a pair of contour (called polygon’s border)
-    and possibly empty sequence of non-overlapping contours
-    which lie within the border (called polygon’s holes).
+    and multicontour which lies within the border (called polygon’s holes).
 
     :param x_coordinates: strategy for vertices' x-coordinates.
     :param y_coordinates:
@@ -946,23 +1248,23 @@ def polygons(x_coordinates: Strategy[Coordinate],
     True
     >>> all(len(vertex) == 2 for hole in holes for vertex in hole)
     True
-    >>> all(all(isinstance(coordinate, coordinates_type)
-    ...         for coordinate in vertex)
-    ...     for vertex in border)
+    >>> all(isinstance(coordinate, coordinates_type)
+    ...     for vertex in border
+    ...     for coordinate in vertex)
     True
-    >>> all(all(isinstance(coordinate, coordinates_type)
-    ...         for coordinate in vertex)
+    >>> all(isinstance(coordinate, coordinates_type)
     ...     for hole in holes
-    ...     for vertex in hole)
+    ...     for vertex in hole
+    ...     for coordinate in vertex)
     True
     >>> all(all(min_coordinate <= coordinate <= max_coordinate
     ...         for coordinate in vertex)
     ...     for vertex in border)
     True
-    >>> all(all(min_coordinate <= coordinate <= max_coordinate
-    ...         for coordinate in vertex)
+    >>> all(min_coordinate <= coordinate <= max_coordinate
     ...     for hole in holes
-    ...     for vertex in hole)
+    ...     for vertex in hole
+    ...     for coordinate in vertex)
     True
 
     For different coordinates' domains:
@@ -1012,14 +1314,14 @@ def polygons(x_coordinates: Strategy[Coordinate],
     True
     >>> all(len(vertex) == 2 for hole in holes for vertex in hole)
     True
-    >>> all(all(isinstance(coordinate, coordinates_type)
-    ...         for coordinate in vertex)
-    ...     for vertex in border)
+    >>> all(isinstance(coordinate, coordinates_type)
+    ...     for vertex in border
+    ...     for coordinate in vertex)
     True
-    >>> all(all(isinstance(coordinate, coordinates_type)
-    ...         for coordinate in vertex)
+    >>> all(isinstance(coordinate, coordinates_type)
     ...     for hole in holes
-    ...     for vertex in hole)
+    ...     for vertex in hole
+    ...     for coordinate in vertex)
     True
     >>> all(min_x_coordinate <= vertex_x <= max_x_coordinate
     ...     and min_y_coordinate <= vertex_y <= max_y_coordinate
@@ -1032,7 +1334,7 @@ def polygons(x_coordinates: Strategy[Coordinate],
     True
     """
     _validate_sizes(min_size, max_size, TRIANGULAR_CONTOUR_SIZE)
-    _validate_sizes(min_holes_size, max_holes_size, EMPTY_HOLES_SIZE,
+    _validate_sizes(min_holes_size, max_holes_size, EMPTY_MULTICONTOUR_SIZE,
                     'min_holes_size', 'max_holes_size')
     _validate_sizes(min_hole_size, max_hole_size, TRIANGULAR_CONTOUR_SIZE,
                     'min_hole_size', 'max_hole_size')
@@ -1069,12 +1371,10 @@ def polygons(x_coordinates: Strategy[Coordinate],
                         max_hole_size: int) -> Strategy[List[int]]:
         if not holes_size:
             return strategies.builds(list)
-        max_hole_size -= holes_size * min_hole_size
         max_holes_sizes = [min_hole_size] * holes_size
         indices = cycle(range(holes_size))
-        while max_hole_size:
+        for _ in range(max_hole_size - holes_size * min_hole_size):
             max_holes_sizes[next(indices)] += 1
-            max_hole_size -= 1
         sizes_ranges = [range(min_hole_size, max_hole_size + 1)
                         for max_hole_size in max_holes_sizes]
         return (strategies.permutations([strategies.sampled_from(sizes_range)
