@@ -6,8 +6,7 @@ from typing import (Callable,
                     List,
                     Optional,
                     Sequence,
-                    Tuple,
-                    TypeVar)
+                    Tuple)
 
 from dendroid import red_black
 from dendroid.hints import Sortable
@@ -21,12 +20,14 @@ from .core.utils import (Orientation,
                          orientation)
 from .hints import (Contour,
                     Coordinate,
+                    Domain,
                     Point,
                     Polygon,
+                    Range,
                     Segment)
 
 
-def to_concave_contour(points: Sequence[Point], size: int) -> Contour:
+def to_contour(points: Sequence[Point], size: int) -> Contour:
     """
     Based on chi-algorithm by M. Duckham et al.
 
@@ -80,7 +81,7 @@ def to_polygon(points: Sequence[Point],
     holes_segments = []
     for hole_size in holes_sizes:
         hole_points = inner_points[start:start + hole_size]
-        hole = to_concave_contour(hole_points, hole_size)[::-1]
+        hole = to_contour(hole_points, hole_size)[::-1]
         holes.append(hole)
         holes_segments.extend(contour_to_segments(hole))
         boundary_vertices.update(hole_points)
@@ -288,10 +289,6 @@ def _to_strict_sub_hull(points: Iterable[Point]) -> List[Point]:
     return result
 
 
-Domain = TypeVar('Domain')
-Range = TypeVar('Range')
-
-
 def pack(function: Callable[..., Range]
          ) -> Callable[[Iterable[Domain]], Range]:
     return partial(apply, function)
